@@ -23,6 +23,7 @@
  */
 
 #include "goxel.h"
+#include "file_format.h"
 
 /*
  * Structure that represents a single voxel and visible faces.
@@ -72,7 +73,7 @@ static void swap_color(uint32_t v, uint8_t ret[4])
     ret[3] = o[3];
 }
 
-static int kv6_import(const char *path)
+static int kv6_import(image_t *image, const char *path)
 {
     FILE *file;
     char magic[4];
@@ -88,10 +89,6 @@ static int kv6_import(const char *path)
         uint8_t visface;
     } *blocks = NULL;
 
-    path = path ?: noc_file_dialog_open(NOC_FILE_DIALOG_OPEN,
-                                        "kv6\0*.kv6\0", NULL, NULL);
-    if (!path) return -1;
-
     file = fopen(path, "rb");
     r = fread(magic, 1, 4, file);
     if (strncmp(magic, "Kvxl ", 4) != 0) raise("Invalid magic");
@@ -144,7 +141,7 @@ static int kv6_import(const char *path)
         }
     }
 
-    mesh_blit(goxel.image->active_layer->mesh, (const uint8_t*)cube,
+    mesh_blit(image->active_layer->mesh, (const uint8_t*)cube,
               -w / 2, -h / 2, -d / 2, w, h, d, NULL);
 end:
     free(cube);
@@ -155,7 +152,7 @@ end:
     return ret;
 }
 
-static int kvx_import(const char *path)
+static int kvx_import(image_t *image, const char *path)
 {
     FILE *file;
     int i, r, ret = 0, nb, size, lastz = 0, len, visface;
@@ -243,9 +240,9 @@ static int kvx_import(const char *path)
     vec3_set(aabb[0], -px, -py, pz - d);
     vec3_set(aabb[1], w - px, h - py, pz);
 
-    bbox_from_aabb(goxel.image->box, aabb);
-    bbox_from_aabb(goxel.image->active_layer->box, aabb);
-    mesh_blit(goxel.image->active_layer->mesh, (uint8_t*)cube,
+    bbox_from_aabb(image->box, aabb);
+    bbox_from_aabb(image->active_layer->box, aabb);
+    mesh_blit(image->active_layer->mesh, (uint8_t*)cube,
               -px, -py, pz - d, w, h, d, NULL);
 
 end:
@@ -313,7 +310,7 @@ static bool slab_append(slab_t *slab, voxel_t *vox)
 }
 
 
-static void kvx_export(const mesh_t *mesh, const char *path)
+static int kvx_export(const image_t *image, const char *path)
 {
     FILE *file;
     uint8_t (*palette)[4];
@@ -331,11 +328,12 @@ static void kvx_export(const mesh_t *mesh, const char *path)
     uint32_t *xyoffsets;
     bool use_current_palette = false;
     float pivot[3];
+    const mesh_t *mesh = goxel_get_layers_mesh(image);
 
     UT_icd voxel_icd = {sizeof(voxel_t), NULL, NULL, NULL};
     UT_icd slab_icd = {sizeof(slab_t), NULL, NULL, NULL};
 
-    mat4_copy(goxel.image->box, box);
+    mat4_copy(image->box, box);
     if (box_is_null(box)) mesh_get_box(mesh, true, box);
 
     size[0] = box[0][0] * 2;
@@ -507,42 +505,18 @@ static void kvx_export(const mesh_t *mesh, const char *path)
     free(xyoffsets);
     free(palette);
     fclose(file);
+    return 0;
 }
 
-static void export_as_kvx(const char *path)
-{
-    path = path ?: sys_get_save_path("kvx\0*.kvx\0", "untitled.kvx");
-    if (!path) return;
-    kvx_export(goxel_get_layers_mesh(), path);
-    sys_on_saved(path);
-}
-
-ACTION_REGISTER(import_kv6,
-    .help = "Import a slab kv6 image",
-    .cfunc = kv6_import,
-    .csig = "vp",
-    .file_format = {
-        .name = "kv6",
-        .ext = "*.kv6\0"
-    },
+FILE_FORMAT_REGISTER(kv6,
+    .name = "kv6",
+    .ext = "slab\0*.kv6\0",
+    .import_func = kv6_import,
 )
 
-ACTION_REGISTER(import_kvx,
-    .help = "Import a slab kvx image",
-    .cfunc = kvx_import,
-    .csig = "vp",
-    .file_format = {
-        .name = "kvx",
-        .ext = "*.kvx\0"
-    },
-)
-
-ACTION_REGISTER(export_as_kvx,
-    .help = "Save the image as a slab kvx image",
-    .cfunc = export_as_kvx,
-    .csig = "vp",
-    .file_format = {
-        .name = "kvx",
-        .ext = "*.kvx\0",
-    },
+FILE_FORMAT_REGISTER(kvx,
+    .name = "kvx",
+    .ext = "slab\0*.kvx\0",
+    .import_func = kvx_import,
+    .export_func = kvx_export,
 )
