@@ -1,3 +1,21 @@
+/* Goxel 3D voxels editor
+ *
+ * copyright (c) 2019 Guillaume Chereau <guillaume@noctua-software.com>
+ *
+ * Goxel is free software: you can redistribute it and/or modify it under the
+ * terms of the GNU General Public License as published by the Free Software
+ * Foundation, either version 3 of the License, or (at your option) any later
+ * version.
+
+ * Goxel is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
+
+ * You should have received a copy of the GNU General Public License along with
+ * goxel.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #ifndef MESH_H
 #define MESH_H
 
@@ -78,6 +96,20 @@ void mesh_delete(mesh_t *mesh);
  */
 void mesh_clear(mesh_t *mesh);
 
+/*
+ * Function: mesh_get_block_aabb
+ * Compute the AABB box of a given block of the mesh.
+ */
+static inline void mesh_get_block_aabb(const int pos[3], int aabb[2][3])
+{
+    aabb[0][0] = pos[0];
+    aabb[0][1] = pos[1];
+    aabb[0][2] = pos[2];
+    aabb[1][0] = pos[0] + BLOCK_SIZE;
+    aabb[1][1] = pos[1] + BLOCK_SIZE;
+    aabb[1][2] = pos[2] + BLOCK_SIZE;
+}
+
 mesh_t *mesh_copy(const mesh_t *mesh);
 
 void mesh_set(mesh_t *mesh, const mesh_t *other);
@@ -106,6 +138,12 @@ void mesh_set_at(mesh_t *mesh, mesh_iterator_t *it,
 // XXX: we should remove this one I guess.
 void mesh_remove_empty_blocks(mesh_t *mesh, bool fast);
 
+/*
+ * Function: mesh_clear_block
+ * Set to zero all the voxels in a given block.
+ */
+void mesh_clear_block(mesh_t *mesh, mesh_iterator_t *it, const int pos[3]);
+
 /*
  * Function: mesh_is_empty
  *
@@ -187,4 +225,12 @@ void mesh_read(const mesh_t *mesh,
                const int pos[3], const int size[3],
                uint8_t *data);
 
+typedef struct {
+    int       nb_meshes;
+    int       nb_blocks;
+    uint64_t  mem;
+} mesh_global_stats_t;
+
+void mesh_get_global_stats(mesh_global_stats_t *stats);
+
 #endif // MESH_H
